==> wristrest-0.1.0.tgz: package/wristrest.js <==
const { MongoClient, ObjectID } = require('mongodb');

let opts, client, db;

const defaultOptions = { db: undefined, pageSize: 20, entityWhitelist: [] };

const sortFromQuery = (req) => {
  let { sort: sortQuery } = req.query;
  delete req.query.sort;
  if (sortQuery) {
    const sort = sortQuery.split(',').reduce((acc, v) => {
      const key = v.startsWith('-') ? v.substring(1) : v;
      acc[key] = v.startsWith('-') ? -1 : 1;
      return acc;
    }, {});
    return { sort, sortQuery }
  }
  return { sort: undefined, sortQuery: undefined };
};

const projectionFromQuery = (req) => {
  let { fields: projectionQuery } = req.query;
  delete req.query.fields;
  if (projectionQuery) {
    const projection = projectionQuery.split(',').reduce((acc, v) => {
      acc[v] = 1;
      return acc;
    }, {});
    return { projection, projectionQuery };
  }
  return { projection: undefined, projectionQuery: undefined };
};

const limitFromQuery = (req) => {
  let { limit } = req.query;
  delete req.query.limit;
  return parseInt(limit, 10) || opts.pageSize;
};

const skipFromQuery = (req) => {
  let { skip } = req.query;
  delete req.query.skip;
  return parseInt(skip, 10) || 0;
};

const linksFromResult = (req, entity, results, projectionQuery, sortQuery, limit, skip) => {
  const nextSkip = skip + limit;
  const prevSkip = skip - limit;
  const fields = projectionQuery ? `&fields=${projectionQuery}` : '';
  const sort = sortQuery ? `&sort=${sortQuery}` : '';
  const links = {};
  if (results.length > limit) {
    links.next = `${req.protocol}://${req.headers.host}/${entity}?limit=${limit}&skip=${nextSkip}${fields}${sort}`
  }
  if (skip > 0) {
    links.prev = `${req.protocol}://${req.headers.host}/${entity}?limit=${limit}&skip=${prevSkip}${fields}${sort}`;
  }
  return links;
};

const transformId = (entities) => {
  return entities.map(e => {
    const _id = e._id;
    delete e._id;
    e.id = _id;
    return e;
  });
};

module.exports = async (options) => {
  opts = Object.assign(defaultOptions, options);

  if (opts.db) {
    db = opts.db;
  } else {
    const dbName = process.env.DB_NAME;
    const dbUri = process.env.DB_URI;
    client = await MongoClient.connect(dbUri, { useNewUrlParser: true });
    db = client.db(dbName);
  }

  return async (req, res, next) => {
    try {
      const method = req.method.toUpperCase();
      const urlSplit = req.originalUrl.split('?').shift().split('/');
      urlSplit.shift(); // remove the leading forward slash
      const entity = urlSplit.shift().toLowerCase();
      const id = urlSplit.shift();

      const whitelist = opts.entityWhitelist || [];
      if (whitelist.length > 0 && !whitelist.includes(entity)) {
        return next();
      }

      if (method === 'GET') {
        const { projection, projectionQuery } = projectionFromQuery(req);
        if (id === undefined) {
          const { sort, sortQuery } = sortFromQuery(req);
          const limit = limitFromQuery(req);
          const skip = skipFromQuery(req);
          const results = transformId(await db.collection(entity).find(req.query, { projection }).sort(sort).limit(limit + 1).skip(skip).toArray());
          const links = linksFromResult(req, entity, results, projectionQuery, sortQuery, limit, skip);
          if (results.length < limit) {
            return res.links(links).json(results);
          }
          results.pop();
          return res.links(links).json(results);
        } else {
          return res.json(transformId(await db.collection(entity).find({ _id: ObjectID.createFromHexString(id) }, { projection }).limit(1).toArray())[0]);
        }
      }

      if (method === 'POST') {
        const now = new Date();
        const doc = Object.assign(req.body, { created_at: now, updated_at: now });
        const result = transformId((await db.collection(entity).insertOne(doc)).ops)[0];
        return res.location(`/${entity}/${result.id}`).status(201).json(result);
      }

      if (method === 'PUT') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        const now = new Date();
        const doc = Object.assign(req.body, { created_at: now, updated_at: now });
        const result = transformId(await db.collection(entity).findOneAndReplace({ _id: ObjectID.createFromHexString(id) }, doc, { upsert: true, returnOriginal: false }));
        return res.json(result);
      }

      if (method === 'PATCH') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        const doc = Object.assign(req.body, { updated_at: new Date() });
        const result = transformId(await db.collection(entity).findOneAndUpdate({ _id: ObjectID.createFromHexString(id) }, { $set: doc }, { upsert: true, returnOriginal: false }));
        return res.json(result);
      }

      if (method === 'DELETE') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        await db.collection(entity).deleteOne({ _id: ObjectID.createFromHexString(id) });
        return res.sendStatus(204);
      }

      next();
    } catch (err) {
      next(err);
    }
  };

};

==> wristrest-0.2.0.tgz: package/wristrest.js <==
const { MongoClient, ObjectID } = require('mongodb');

let opts, db;

const defaultOptions = { pageSize: 20 };

const sortFromQuery = (req) => {
  let { sort: sortQuery } = req.query;
  delete req.query.sort;
  if (sortQuery) {
    const sort = sortQuery.split(',').reduce((acc, v) => {
      const key = v.startsWith('-') ? v.substring(1) : v;
      acc[key] = v.startsWith('-') ? -1 : 1;
      return acc;
    }, {});
    return { sort, sortQuery }
  }
  return { sort: undefined, sortQuery: undefined };
};

const projectionFromQuery = (req) => {
  let { fields: projectionQuery } = req.query;
  delete req.query.fields;
  if (projectionQuery) {
    const projection = projectionQuery.split(',').reduce((acc, v) => {
      acc[v] = 1;
      return acc;
    }, {});
    return { projection, projectionQuery };
  }
  return { projection: undefined, projectionQuery: undefined };
};

const limitFromQuery = (req) => {
  let { limit } = req.query;
  delete req.query.limit;
  return parseInt(limit, 10) || opts.pageSize;
};

const skipFromQuery = (req) => {
  let { skip } = req.query;
  delete req.query.skip;
  return parseInt(skip, 10) || 0;
};

const linksFromResult = (req, entity, results, projectionQuery, sortQuery, limit, skip) => {
  const nextSkip = skip + limit;
  const prevSkip = skip - limit;
  const fields = projectionQuery ? `&fields=${projectionQuery}` : '';
  const sort = sortQuery ? `&sort=${sortQuery}` : '';
  const links = {};
  if (results.length > limit) {
    links.next = `${req.protocol}://${req.headers.host}/${entity}?limit=${limit}&skip=${nextSkip}${fields}${sort}`
  }
  if (skip > 0) {
    links.prev = `${req.protocol}://${req.headers.host}/${entity}?limit=${limit}&skip=${prevSkip}${fields}${sort}`;
  }
  return links;
};

const transformId = (entities) => {
  return entities.map(e => {
    const _id = e._id;
    delete e._id;
    e.id = _id;
    return e;
  });
};

const connectToDatabase = async () => {
  if (!db) {
    const dbName = process.env.DB_NAME;
    const dbUri = process.env.DB_URI;
    const client = await MongoClient.connect(dbUri, { useNewUrlParser: true });
    db = client.db(dbName);
  }
};

module.exports = (options) => {
  opts = Object.assign(defaultOptions, options);
  return async (req, res, next) => {
    try {
      await connectToDatabase();

      const method = req.method.toUpperCase();
      const urlSplit = req.originalUrl.split('?').shift().split('/');
      urlSplit.shift(); // remove the leading forward slash
      const entity = urlSplit.shift().toLowerCase();
      const id = urlSplit.shift();

      if (method === 'GET') {
        const { projection, projectionQuery } = projectionFromQuery(req);
        if (id === undefined) {
          const { sort, sortQuery } = sortFromQuery(req);
          const limit = limitFromQuery(req);
          const skip = skipFromQuery(req);
          const filter = req.query;
          if (req.userId) {
            filter.userId = req.userId;
          }
          const results = transformId(await db.collection(entity).find(filter, { projection }).sort(sort).limit(limit + 1).skip(skip).toArray());
          const links = linksFromResult(req, entity, results, projectionQuery, sortQuery, limit, skip);
          if (results.length < limit) {
            return res.links(links).json(results);
          }
          results.pop();
          return res.links(links).json(results);
        } else {
          const filter = { _id: ObjectID.createFromHexString(id) };
          if (req.userId) {
            filter.userId = req.userId;
          }
          return res.json(transformId(await db.collection(entity).find(filter, { projection }).limit(1).toArray())[0]);
        }
      }

      if (method === 'POST') {
        const now = new Date();
        const doc = Object.assign(req.body, { created_at: now, updated_at: now });
        if (req.userId) {
          doc.userId = req.userId;
        }
        const result = transformId((await db.collection(entity).insertOne(doc)).ops)[0];
        return res.location(`/${entity}/${result.id}`).status(201).json(result);
      }

      if (method === 'PUT') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        const now = new Date();
        const doc = Object.assign(req.body, { created_at: now, updated_at: now });
        const filter = { _id: ObjectID.createFromHexString(id) };
        if (req.userId) {
          doc.userId = req.userId;
          filter.userId = req.userId;
        }
        const result = transformId(await db.collection(entity).findOneAndReplace(filter, doc, { upsert: true, returnOriginal: false }));
        return res.json(result);
      }

      if (method === 'PATCH') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        const doc = Object.assign(req.body, { updated_at: new Date() });
        const filter = { _id: ObjectID.createFromHexString(id) };
        if (req.userId) {
          doc.userId = req.userId;
          filter.userId = req.userId;
        }
        const result = transformId(await db.collection(entity).findOneAndUpdate(filter, { $set: doc }, { upsert: true, returnOriginal: false }));
        return res.json(result);
      }

      if (method === 'DELETE') {
        if (id === undefined) {
          return res.sendStatus(400);
        }
        const filter = { _id: ObjectID.createFromHexString(id) };
        if (req.userId) {
          filter.userId = req.userId;
        }
        await db.collection(entity).deleteOne(filter);
        return res.sendStatus(204);
      }

      next();
    } catch (err) {
      next(err);
    }
  };

};
